/* 
 * Drone control system
 *
 */

#include "definitions.h"
#include <arduino/usarts.h>

// Pin layout
#define FLIGHTMODE_PIN 6 // PWM6
#define YAW_PIN 7 // PWM7
#define PITCH_PIN 11 // PWM11
#define ROLL_PIN 12 // PWM12
#define THROTTLE_PIN 13 // PWM13

#define PWM_CENTER 1500

//						Element number	0		1		2		3		4		5		6		7		8		9		10
// Pitch and roll speed
static int pitch_value[11] = 		{ 	1340,	1370, 	1400, 	1440, 	1460, 	1500, 	1540, 	1560, 	1600, 	1630, 	1660	};
static int roll_value[11] = 		{ 	1340,	1370, 	1400, 	1440, 	1460, 	1500, 	1540, 	1560, 	1600, 	1630, 	1660	};

// Flight height: Ground (0), 70 cm (1500), 80 cm (1600)
static int throttle_value[3] =  	{ 	1100, 	1500,  	1600	};

// Flightmode: LAND (1900), FM1 (1500), FM2 (1100)
static int flightmode_value[3] = 	{ 	1900, 	1500, 	1100	};

// Rudder function in different flight mode:
//  LAND: Rudder works as config setup - TRIM (1100) or EMERG (1900)
//  FM1/FM2: Rudder works as yaw speed
static int rudder_value[11] = 		{ 	1100, 	1300, 	1400, 	1420, 	1440, 	1500, 	1560, 	1580, 	1600, 	1700,	1900	};

static void pwm_init(void)
{
    // Set PWM pins as output
	pinMode(PITCH_PIN, OUTPUT); pinMode(ROLL_PIN, OUTPUT); pinMode(THROTTLE_PIN, OUTPUT);
	pinMode(YAW_PIN, OUTPUT); pinMode(FLIGHTMODE_PIN, OUTPUT);

    // Reset Timer/counter 1
	TCCR1A = 0;
	TCCR1B = 0;
	TCCR1C = 0;
	TCCR4A = 0;
	TCCR4B = 0;
	TCCR4C = 0;

    // Set Timer/counter 1 to count to 20000 ( 0 - 19999). (with prescaler it would be 50Hz)
	ICR1 = 19999;
	ICR4 = 19999;

    // Set Phase and Freqency correct that count up from 0 to ICR1, then down to 0 again
	TCCR1A &= !(1<<WGM11) | !(1<<WGM10);
	TCCR1B &= !(1<<WGM12);
	TCCR4A &= !(1<<WGM11) | !(1<<WGM10);
	TCCR4B &= !(1<<WGM12);
	// Set Prescaler to 8
	TCCR1B |= (1<<WGM13) | (1<<CS11);
	TCCR4B |= (1<<WGM13) | (1<<CS11);

    // Set output compare mode for Timer/counter 1. (OCR1A will decide the pulse width)
	TCCR1A |= (1<<COM1A1);
	TCCR1A |= (1<<COM1B1);
	TCCR1A |= (1<<COM1C1);
	TCCR4A |= (1<<COM4A1);
	TCCR4A |= (1<<COM4B1);
	TCCR4A |= (1<<COM4C1);

	// Set Pulse width to  1.5 ms, center position
	OCR1A = PWM_CENTER;
	OCR1B = PWM_CENTER;
	OCR1C = PWM_CENTER;
	OCR4A = PWM_CENTER;
	OCR4B = PWM_CENTER;
	OCR4C = PWM_CENTER;
}

static void set_flightmode (int val)
{
    // Set Pulse width to value
	OCR4A = val;
}

static void set_yaw (int val)
{
    // Set Pulse width to value
	OCR4B = val;
}

static void set_pitch (int val)
{
    // Set Pulse width to value
	OCR1A = val;
}

static void set_roll (int val)
{
    // Set Pulse width to value
	OCR1B = val;
}

static void set_throttle (int val)
{
    // Set Pulse width to value
	OCR1C = val;
}

void control(void)
{
	static int init = 0;
	static int control_value = 0;
	static ControlcommandType control_cmds = 0;
	//ColorType color;

	// Initialize
	if (!init)
	{
    	pwm_init();
		if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Control - init\r\n");

		// set default control parameters
		set_throttle(throttle_value[0]); // Default 1100
		set_roll(roll_value[5]); // Default 1500
		set_pitch(pitch_value[5]); // Default 1500
		set_yaw(rudder_value[5]); // Default  1500
		set_flightmode(flightmode_value[0]); // Default 1900

		init = 1;
	}

	if (receivedSignal(control_command, &control_cmds, &control_value))
	{
			if (DEBUG && CONTROL_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Control - received control_cmd %d value %d\r\n", control_cmds, control_value);
			switch(control_cmds)
			{
				case CC_Pitch:
					set_pitch(pitch_value[control_value]);
					break;
				case CC_Roll:
					set_roll(roll_value[control_value]);
					break;
				case CC_Throttle:
					set_throttle(throttle_value[control_value]);
					break;
				case CC_Flightmode:
					set_flightmode(flightmode_value[control_value]);
					break;
				case CC_Yaw:
					set_yaw(rudder_value[control_value]);
					break;
			}

	}
} //end control
