/*
 * Line and symbol correction process
 *
 */

#include "definitions.h"

// Correction zones linear - x and y coordinates
#define XY_OFFSET_CHANGE 200
#define XY_OFFSET_CHANGE_NEG -200
#define XY_OFFSET_APPROVED 100
#define XY_OFFSET_APPROVED_NEG -100

// Correction speeds
#define SPEED_HIGHER_VALUE_1 6 // a low speed pitch to the left
#define SPEED_HIGHER_VALUE_2 7 // a little faster pitch to the left
#define SPEED_LOWER_VALUE_1 4 // a low speed pitch to the right
#define SPEED_LOWER_VALUE_2 3 // a little faster pitch to the right
#define SPEED_CENTER_VALUE 5

// Correction zones rotation - angular correction
#define ANGLE_CHANGE M_PI/8      		// 22.5 degree
#define ANGLE_CHANGE_NEG -M_PI/8		//-22.5 degree
#define ANGLE_APPROVED M_PI/32			// 5.625 degree
#define ANGLE_APPROVED_NEG -M_PI/32		//-5.625 degree

// Find the needed correction control speed based on x-axis offset
int x_calculation(double x_value)
{
	static int x_speed;


	if((x_value > XY_OFFSET_CHANGE))
	{
		// In zone 2, max speed in x correction
		x_speed = SPEED_HIGHER_VALUE_2;
	}
	else if((x_value > XY_OFFSET_APPROVED))
	{
		// In zone 1, min speed in x correction
		x_speed = SPEED_HIGHER_VALUE_1;
	}
	else if((x_value < XY_OFFSET_CHANGE_NEG))
	{
		// In zone neg 2, max speed in x correction
		x_speed = SPEED_LOWER_VALUE_2;
	}
	else if((x_value < XY_OFFSET_APPROVED_NEG))
	{
		// In zone neg 1, min speed in x correction
		x_speed =SPEED_LOWER_VALUE_1;
	}
	else
	{
		// In zone 0, no x correction needed
		x_speed = SPEED_CENTER_VALUE;
	}

	return x_speed;
}

// Find the needed correction control speed based on y-axis offset
int y_calculation(double y_value)
{
	static int y_speed;


	if((y_value > XY_OFFSET_CHANGE))
	{
		// In zone 2, max speed in y correction
		y_speed = SPEED_LOWER_VALUE_2;
	}
	else if((y_value > XY_OFFSET_APPROVED))
	{
		// In zone 1, min speed in y correction
		y_speed = SPEED_LOWER_VALUE_1;
	}
	else if((y_value < XY_OFFSET_CHANGE_NEG))
	{
		// In zone neg 2, max speed in y correction
		y_speed = SPEED_HIGHER_VALUE_2;
	}
	else if((y_value < XY_OFFSET_APPROVED_NEG))
	{
		// In zone neg 1, min speed in y correction
		y_speed = SPEED_HIGHER_VALUE_1;
	}
	else
	{
		// In zone 0, no y correction needed
		y_speed = SPEED_CENTER_VALUE;
	}

	return y_speed;
}


// Find the needed correction control speed based on angular offset
int angle_calculation(double angle_value)
{
	static int angle_speed;
	// Set the correct turning speed
	if (angle_value > ANGLE_CHANGE)
	{
		//over 22.5 degree wrong
		angle_speed = SPEED_HIGHER_VALUE_2;
	}
	else if (angle_value > ANGLE_APPROVED)
	{
		// Over 5.625 degree wrong
		angle_speed = SPEED_HIGHER_VALUE_1;
	}
	else if (angle_value < ANGLE_CHANGE_NEG)
	{
		// Over -22.5 degree wrong
		angle_speed = SPEED_LOWER_VALUE_2;
	}
	else if (angle_value < ANGLE_APPROVED_NEG)
	{
		// Over -5.625 degree wrong
		angle_speed = SPEED_LOWER_VALUE_1;
	}
	else
	{
		// Within approved angle
		angle_speed = SPEED_CENTER_VALUE;
	}
	return angle_speed;
}

void correction(void)
{
	static int init = 0;
	static double angle, x_offset, y_offset;
	static int angle_speed, old_angle_speed, x_speed, old_x_speed, y_speed, old_y_speed;
	static ControlcommandType control_cmd; //CC_Pitch, CC_Roll, CC_Throttle, CC_Flightmode, CC_Rudder

	// Initialize
	if (!init)
	{
		if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Correction - init\r\n");
		init = 1;
	}

	//if (DEBUG) usartPrint(DEBUG_PORT,"Correction - corr_state %s", debug_corr[corr_state]);

	// Line correction -  x-axis, angular used
	if (receivedSignal(line_correction_data, &angle, &x_offset))
	{

		x_speed = x_calculation(x_offset);
		angle_speed = angle_calculation(angle);

		// x-axis roll correction
		if (x_speed != old_x_speed)
		{
			// Signal with the correct information
			control_cmd = CC_Roll;
			if (DEBUG && CORR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Correction - Line x roll correction, x speed %d\r\n", x_speed);
			sendSignal(control_command, &control_cmd, &x_speed);
			old_x_speed = x_speed;
		}

		// Yaw rotation correction
		if (angle_speed != old_angle_speed)
		{
			// Signal with the correct information
			control_cmd = CC_Yaw;
			if (DEBUG && CORR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Correction - Line yaw correction, angle speed %d\r\n", angle_speed);
			sendSignal(control_command, &control_cmd, &angle_speed);
			old_angle_speed = angle_speed;
		}
	}
	// Symbol correction - x-axis, y-axis, angular used
	else if (receivedSignal(symbol_correction_data,&angle, &x_offset, &y_offset))
	{
		x_speed = x_calculation(x_offset);
		y_speed = y_calculation(y_offset);
		angle_speed = angle_calculation(angle);

		// x-axis roll correction
		if (x_speed != old_x_speed)
		{
			// Signal with the correct information
			control_cmd = CC_Roll;
			if (DEBUG && CORR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Correction - Symbol x roll correction, x speed %d\r\n", x_speed);
			sendSignal(control_command, &control_cmd, &x_speed);
			old_x_speed = x_speed;
		}

		// y-axis pitch correction
		if (y_speed != old_y_speed)
		{
			// Signal with the correct information
			control_cmd = CC_Pitch;
			if (DEBUG && CORR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Correction - Symbol y pitch correction, y speed %d\r\n", x_speed);
			sendSignal(control_command, &control_cmd, &y_speed);
			old_y_speed = y_speed;
		}

		// Yaw rotation correction
		if (angle_speed != old_angle_speed)
		{
			// Signal with the correct information
			control_cmd = CC_Yaw;
			if (DEBUG && CORR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Correction - Symbol yaw correction, angle speed %d\r\n", angle_speed);
			sendSignal(control_command, &control_cmd, &angle_speed);
			old_angle_speed = angle_speed;
		}

		// We're correctly positioned NOTE: need fill in values for not moving
		if ((x_speed == 5) && (y_speed == 5 ) && (angle_speed == 5))
		{
			if (DEBUG && CORR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Correction - Symbol correction done\r\n");
			sendSignal(symbol_correction_done);
		}
	}
}
