/*
 * IR camera process
 *
 */

#include "definitions.h"
#include <arduino/usarts.h>

// I2C address for the device. Note: only 7-bit for the address, LSB for read/write control later on
#define I2C_ADDR 0xB0

// Bit flags for blobs
#define BLOB1 0x01
#define BLOB2 0x02
#define BLOB3 0x04
#define BLOB4 0x08

// Update frequency - 100 Hz
#define BLOB_TIMER 30

// Up to four points can be tracked
Blob blob[4];

static void ircamera_init(void)
{
	// Initialize the sensor by enabling, setting sensitivity and operation mode
	if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: IR Camera - I2C init process\r\n");

    // Initialize i2c library
    i2c_init();
    if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: IR Camera - I2C lib done\r\n");

    //               Block 1                     Block 2
    // Register addr 00 01 02 03 04 05 06 07 08  1A 1B
    // Values        00 00 00 00 00 00 90 00 C0  40 00  Suggested by Marcan
    // 				 00 00 00 00 00 00 90 00 41	 40 00	Suggested by inio (max sensitivity)
    // 				 02 00 00 71 01 00 64 00 fe	 fd 05	Wii level 1 (least sensitivity, max resolution)
    // 				 02 00 00 71 01 00 96 00 b4	 b3 04	Wii level 2
    // 				 02 00 00 71 01 00 aa 00 64	 63 03	Wii level 3
    // 				 02 00 00 71 01 00 c8 00 36	 35 03	Wii level 4 - works ok
    // 				 07 00 00 71 01 00 72 00 20	 1f 03	Wii level 5 (max sensitivity, least resolution)
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x30); i2c_write(0x01); i2c_stop(); delay(100); // Camera enable
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x30); i2c_write(0x08); i2c_stop(); delay(100); // Acknowledge

    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x00); i2c_write(0x00); i2c_stop(); delay(100); // Sensitivity block 1 byte part 1 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x01); i2c_write(0x00); i2c_stop(); delay(100); // Sensitivity block 1 byte part 1 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x02); i2c_write(0x00); i2c_stop(); delay(100); // Sensitivity block 1 byte part 1 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x03); i2c_write(0x00); i2c_stop(); delay(100); // Sensitivity block 1 byte part 1 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x04); i2c_write(0x00); i2c_stop(); delay(100); // Sensitivity block 1 byte part 1 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x05); i2c_write(0x00); i2c_stop(); delay(100); // Sensitivity block 1 byte part 1 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x06); i2c_write(0x90); i2c_stop(); delay(100); // Sensitivity block 1 byte part 1 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x07); i2c_write(0x00); i2c_stop(); delay(100); // Sensitivity block 1 byte part 1 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x08); i2c_write(0xc0); i2c_stop(); delay(100); // Sensitivity block 1 byte part 2 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x1A); i2c_write(0x40); i2c_stop(); delay(100); // Sensitivity block 2 - Marcan
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x1B); i2c_write(0x00); i2c_stop(); delay(100); // Sensitivity block 2 - Marcan

    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x33); i2c_write(0x03); i2c_stop(); delay(100); // Set mode number 3 - Extended data (blob size too)
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x30); i2c_write(0x08); i2c_stop(); delay(100); // Acknowledge
    if (INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: IR Camera - I2C init done\r\n");
}

static void ircamera_getdata (int blobs[])
{
    unsigned int ir_data[16];
    unsigned int s, i;
    static char init = 0;

    
    if (!init) {
        // Port setup, outputs and pull-up on SDA, SCL, and LED-pin
        pinMode(19, OUTPUT); pinMode(20, OUTPUT); pinMode(21, OUTPUT);
        digitalWrite(19, HIGH);
        // We use our own pull-up resistors, no need for internal pullup
        // digitalWrite(20, HIGH); digitalWrite(21, HIGH);

        // Turn on LED
        digitalWrite(19, HIGH);

        // Initilize i2c and sensor
        if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: IR Camera - init\r\n");

        i2c_init();
        delay(100);
        ircamera_init();

        init = 1;
    }

    // Start IR camera sensor to read optical array
    //if (DEBUG) usartPrint(DEBUG_PORT,21000);//"IR camera - Read sensor data";
    //i2c_writebyte(I2C_ADDR,0x36,0x00);

    //if (DEBUG) usartPrint(DEBUG_PORT,22000);//"IR camera - Reading values..";
    
    // Request sensor data
    i2c_start_wait(I2C_ADDR+I2C_WRITE); i2c_write(0x36); i2c_stop();
    i2c_start_wait(I2C_ADDR+I2C_READ);

    // In extended mode there are 3 bytes per blob, 12 total, starting at byte 1 (not 0)
    for(i = 0; i < 12+1; ++i){
    	ir_data[i] = i2c_readAck();
    	//usartPrint(0,"\n\rData byte %d: %d",i,i2c_readAck());
    }
    i2c_stop();

    // Arrange ir data to proper blob structure
    blob[0].x = ir_data[1];
    blob[0].y = ir_data[2];
    s   = ir_data[3];
    blob[0].x += (s & 0x30) <<4;
    blob[0].y += (s & 0xC0) <<2;
    blob[0].size = (s & 0x0F);

    if (blob[0].size > 14) {
        blob[0].x = -1; blob[0].y = -1;
    }
    
    blob[1].x = ir_data[4];
    blob[1].y = ir_data[5];
    s   = ir_data[6];
    blob[1].x += (s & 0x30) <<4;
    blob[1].y += (s & 0xC0) <<2;
    blob[1].size = (s & 0x0F);

    if (blob[1].size > 14) {
        blob[1].x = -1; blob[1].y = -1;
    }

    blob[2].x = ir_data[7];
    blob[2].y = ir_data[8];
    s   = ir_data[9];
    blob[2].x += (s & 0x30) <<4;
    blob[2].y += (s & 0xC0) <<2;
    blob[2].size = (s & 0x0F);

    if (blob[2].size > 14) {
        blob[2].x = -1; blob[2].y = -1;
    }

    blob[3].x = ir_data[10];
    blob[3].y = ir_data[11];
    s   = ir_data[12];
    blob[3].x += (s & 0x30) <<4;
    blob[3].y += (s & 0xC0) <<2;
    blob[3].size = (s & 0x0F);

    if (blob[3].size > 14) {
        blob[3].x = -1; blob[3].y = -1;
    }

    // Put blobs into existing passed array
    //strcpy(blobs[0],blob1);
    //blobs[1] = &blob2;
    //blobs[2] = &blob3;
    //blobs[3] = &blob4;

    // Debug
    if (blob[0].size < 15) if (DEBUG && IR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: IR Camera - Blob1: x %d y %d size %d\r\n",blob[0].x,blob[0].y,blob[0].size);
    if (blob[1].size < 15) if (DEBUG && IR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: IR Camera - Blob2: x %d y %d size %d\r\n",blob[1].x,blob[1].y,blob[1].size);
    if (blob[2].size < 15) if (DEBUG && IR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: IR Camera - Blob3: x %d y %d size %d\r\n",blob[2].x,blob[2].y,blob[2].size);
    if (blob[3].size < 15) if (DEBUG && IR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: IR Camera - Blob4: x %d y %d size %d\r\n",blob[3].x,blob[3].y,blob[3].size);

    // Send blob data to symbol reconizer
    if (blob[0].size < 15 || blob[1].size < 15 || blob[2].size < 15 || blob[3].size < 15)
    	sendSignal(blob_data, &blob[0].x, &blob[0].y, &blob[0].size,  &blob[1].x, &blob[1].y, &blob[1].size,
    				&blob[2].x, &blob[2].y, &blob[2].size, &blob[3].x, &blob[3].y, &blob[3].size);

}

void ircamera(void)
{
	static Timer t1;
	static int init = 0;
	int blobs[4];
	//Blob blob1;

	// Initialize
	if (!init)
	{
		t1 = newTimer();
		startTimer(BLOB_TIMER, t1);
		init = 1;
	}

	if (timeout(t1))
	{
		startTimer(BLOB_TIMER, t1);
		if (DEBUG && IR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: IR Camera - Getting blob data...\n\r");
		ircamera_getdata(blobs);
	}

} // end ircamera
