/*
 * Sensor monitor process
 *
 * Sensors   S - Sensor, A - Analog, D - Digital, # - Number
 *
 * Code  channel	Pin				Description
 *
 * SA0 - 0			Analog in 0		Object in objectbox
 * SA1 - 1			Analog in 1		Battery voltage sensor
 *
 */

#include "definitions.h"

// Sensor pull update time
#define CHECK_CYCLE_TIME 100

// Number of channels to install
#define NUM_CH 2

// Analog sensor pin layout
#define SA0_CH 0
#define SA1_CH 1

// Analog voltage thresholds
#define SA0_THRESH  700
#define	BATTERY_LOW 400		// value of battery voltage, for charging
#define BATTERY_FULL 500	// battery fully charged

static int distance_check (int ch_no, int threshold)
{
	// Read voltage on distance sensor
	if (adcRead(ch_no) > threshold)
	{
		return 1;
	}
	return 0;
}

static char sa0_sensor(void)
{
	static int distance = 0, was_distance = 0;

	// Get current value from sensor
	distance = distance_check(SA0_CH, SA0_THRESH);

	// New object in box on top of the drone
	if (distance && (distance != was_distance))
	{
		was_distance = distance; // 1
		return 1;
	}

	// Object left the box
	else if ((distance != was_distance))
	{
		was_distance = distance; // 0
		return 2;
	}

	// No change
	return 0;
}

static char voltage_sensor(char reset)
{
	static int current_voltage;
	static char flag = 0;

	current_voltage = adcRead(SA1_CH);

	// Battery voltage has gone below low threshold
	if(!flag && (current_voltage <= BATTERY_LOW))
	{
		flag = 1;
		return 1;
	}
	// Battery has been fully charged, above threshold
	else if (flag && (current_voltage >= BATTERY_FULL))
	{
		flag = 0;
		return 2;
	}
	if (reset) flag = 0;

	// No change
	return 0;
}

void sensormonitor(void)
{
	static int init = 0;
	static Timer check_timer;
	static char bc_reset;

	// Initialize
	if (!init)
	{
		adcInstall(NUM_CH); // install ADC
		check_timer = newTimer();
		startTimer(CHECK_CYCLE_TIME, check_timer);
		if (DEBUG && INIT_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Sensor monitor - init\r\n");
		bc_reset = 0;
		init = 1;
	}

	// Sensor check
	if (timeout(check_timer))
	{
		startTimer(CHECK_CYCLE_TIME, check_timer);

		switch(sa0_sensor())
		{
			case 0:
				//No change
				break;
			case 1:
				if (DEBUG && SENSOR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Sensormonitor - Object in box\r\n");
				//sendSignal(object_in_box);
				//emptySignalQueue(no_object_in_box);
				break;
			case 2:
				//sendSignal(no_object_in_box);
				//emptySignalQueue(object_in_box);
				if (DEBUG && SENSOR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Sensormonitor - No object in box\r\n");
				break;
		}

		switch(voltage_sensor(bc_reset))
		{
			case 0:
				// No change
				break;
			case 1:
				//sendSignal(charging_needed);
				if (DEBUG && SENSOR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Sensormonitor - Voltage low, charging needed\r\n");
				bc_reset = 0;
				break;
			case 2:
				//sendSignal(charging_done);
				if (DEBUG && SENSOR_DEBUG) usartPrint(DEBUG_PORT,"DEBUG: Sensormonitor - Charging done\r\n");
				bc_reset = 0;
				break;
		}
	}

	// If start signal is received from console, cancel charging and take-off
	else if (receivedSignal(reset_batterycheck))
	{
		bc_reset = 1;
	}

} // end sensormonitor
