/* 
 * Stepper motor process for the belt system
 *
 * Full-step - more strength and torque
 *  A single step is 1.8deg (cycle) consisting of a total of 4 operations, one revolution 200 steps
 * Half-step - accurate, mix of wave drive and full step mode
 *  A single step is 0.9deg (cycle) consisting of a total of 8 operations, one revolution 400 step
 * 
 * char steps[] = {0b1000,0b0100,0b0010,0b0001}; // wavedrive
 * char steps[] = {0b1001,0b1010,0b0110,0b0101}; // full-step
 * char steps[] = {0b1000,0b1010,0b0010,0b0110,0b0100,0b0101,0b0001,0b1001}; // half-step
 *
 */

#include "definitions.h"

// Waiting period before disengaging the stepper motor
#define STOP_DELAY 600

// Timing delay to adjust the stepper motor speed
#define BELT_SPEED 3

void stepper_belt(void)
{
	static int belt_motorpins[] = {2, 3, 4, 5};
	static char steps[] = {0b0101, 0b0110, 0b1010, 0b1001}; // full-step
	static char seq[] = {0b1000, 0b0100, 0b0010, 0b0001}; // a single cycle sequence
	static enum {still, running} current_operation = still;
	static Timer stoptimer;
	static char stop_sequence = 0;

	static int i = 0;
	static Timer t1;
	static int init = 0;

	// Initialize
	if (!init)
	{
		if (DEBUG && INIT_DEBUG) usartWriteString_328(DEBUG_PORT, "BS - init\r\n");
		t1 = newTimer_328();
		stoptimer = newTimer_328();
		//slowstartincrease=newTimer();
		for (int j = 0; j < 4; j++) // pins
		{
			pinMode_328(belt_motorpins[j], OUTPUT);
			digitalWrite_328(belt_motorpins[j], 0);
		}
		init = 1;
	}

	switch (current_operation)
	{
		case still:

			if (receivedSignal(belt_stop))
			{
				//if (DEBUG && BELTSTEPPER_DEBUG) usartWriteString_328(DEBUG_PORT, "BS - Still - Received Stop\r\n");
				stop_sequence = 1;
				startTimer_328(STOP_DELAY, stoptimer);
			}
			else if (receivedSignal(belt_start))
			{
				//if (DEBUG && BELTSTEPPER_DEBUG) usartWriteString_328(DEBUG_PORT, "BS - Still - Received Start\r\n");
				startTimer_328(BELT_SPEED, t1);
				current_operation = running;
			}
			//wait a little while for motor to set pins to 0,0,0,0
			else if (timeout_328(stoptimer) && stop_sequence)
			{
				for (int j = 0; j < 4; j++) // pins
				{
					digitalWrite_328(belt_motorpins[j], 0);
				}
				stop_sequence = 0;
			}

			break;

		case running:
			if (receivedSignal(belt_stop))
			{
				//if (DEBUG && BELTSTEPPER_DEBUG) usartWriteString_328(DEBUG_PORT, "BS - Running - Received Stop\r\n");
				stop_sequence = 1;
				startTimer_328(STOP_DELAY, stoptimer);
				current_operation = still;
			}
			else if (receivedSignal(belt_start))
			{

			}
			// Continuously keep moving the motor
			else if (timeout_328(t1))
			{

				startTimer_328(BELT_SPEED, t1);
				for (int j = 0; j < 4; j++) // pins
				{
					digitalWrite_328(belt_motorpins[j], (steps[i] & seq[j]));
				}
				if (++i > (4 - 1)) i = 0;
			}
			break;
	}
} // end beltstepper
