/*
 * Stepper motor process for the magnet
 *
 * Full-step - more strength and torque
 *  A single step is 1.8deg (cycle) consisting of a total of 4 operations, one revolution 200 steps
 * Half-step - accurate, mix of wave drive and full step mode
 *  A single step is 0.9deg (cycle) consisting of a total of 8 operations, one revolution 400 step
 *
 * char steps[] = {0b1000,0b0100,0b0010,0b0001}; // wavedrive
 * char steps[] = {0b1001,0b1010,0b0110,0b0101}; // full-step
 * char steps[] = {0b1000,0b1010,0b0010,0b0110,0b0100,0b0101,0b0001,0b1001}; // half-step
 *
 */

#include "definitions.h"

// Waiting period before disengaging the stepper motor
#define STOP_DELAY 600

// Timing delay to adjust the stepper motor speed
#define MAGNET_SPEED 2

void stepper_magnet(void)
{
	static int magnet_motorpins[] = {10, 11, 12, 13};
	static char steps[] = {0b0101, 0b0110, 0b1010, 0b1001}; // full-step
	static char seq[] = {0b1000, 0b0100, 0b0010, 0b0001}; // a single cycle sequence
	static enum {still, in, out} current_operation = still;
	static Timer stoptimer;
	static char stop_sequence = 0;

	static int i = 0;
	static Timer t1;
	static int init = 0;

	// Initialize
	if (!init)
	{
		//if (DEBUG && INIT_DEBUG) usartWriteString_328(DEBUG_PORT, "MS - init\r\n");
		t1 = newTimer_328();
		stoptimer = newTimer_328();
		//slowstartincrease=newTimer();
		for (int j = 0; j < 4; j++) // pins
		{
			pinMode_328(magnet_motorpins[j], OUTPUT);
			digitalWrite_328(magnet_motorpins[j], 0);
		}
		//if (DEBUG) debug_msg(9); // "Stepper left - init";
		init = 1;
	}

	switch (current_operation)
	{
		case still:

			if (receivedSignal(magnet_stop))
			{
				//if (DEBUG && MAGNETSTEPPER_DEBUG) usartWriteString_328(DEBUG_PORT, "MS - Still - Received stop\r\n");
				stop_sequence = 1;
				startTimer_328(STOP_DELAY, stoptimer);
			}
			else if (receivedSignal(magnet_in))
			{
				//if (DEBUG && MAGNETSTEPPER_DEBUG) usartWriteString_328(DEBUG_PORT, "MS - Still - Received move in\r\n");
				startTimer_328(MAGNET_SPEED, t1);
				current_operation = in;
			}
			else if (receivedSignal(magnet_out))
			{
				//if (DEBUG && MAGNETSTEPPER_DEBUG) usartWriteString_328(DEBUG_PORT, "MS - Still - Received move out\r\n");
				startTimer_328(MAGNET_SPEED, t1);
				current_operation = out;
			}
			//wait a little while for motor to set pins to 0,0,0,0
			else if (timeout_328(stoptimer) && stop_sequence)
			{
				for (int j = 0; j < 4; j++) // pins
				{
					digitalWrite_328(magnet_motorpins[j], 0);
				}
				stop_sequence = 0;
			}

			break;

		case in:
			if (receivedSignal(magnet_stop))
			{
				//if (DEBUG && MAGNETSTEPPER_DEBUG) usartWriteString_328(DEBUG_PORT, "MS - In - Received stop\r\n");
				stop_sequence = 1;
				startTimer_328(STOP_DELAY, stoptimer);
				current_operation = still;
			}
			// Continuously keep moving the motor
			else if (timeout_328(t1))
			{

				startTimer_328(MAGNET_SPEED, t1);
				for (int j = 0; j < 4; j++) // pins
				{
					digitalWrite_328(magnet_motorpins[j], (steps[i] & seq[j]));
				}
				if (++i > (4 - 1)) i = 0;
			}
			break;

		case out:
			if (receivedSignal(magnet_stop))
			{
				//if (DEBUG && MAGNETSTEPPER_DEBUG) usartWriteString_328(DEBUG_PORT, "MS - Out - Received stop\r\n");
				stop_sequence = 1;
				startTimer_328(STOP_DELAY, stoptimer);
				current_operation = still;
			}

			// Continuously keep moving the motor
			else if (timeout_328(t1))
			{
				startTimer_328(MAGNET_SPEED, t1);
				for (int j = 0; j < 4; j++) // pins
				{
					digitalWrite_328(magnet_motorpins[j], (steps[i] & seq[j]));
				}
				if (--i < 0) i = (4 - 1);
			}
			break;
	}
} // end stepper right
