/*
 * Sensor monitor process
 *
 * Sensors   S - Sensor, A - Analog, D - Digital, # - Number
 *
 * Code Channel PIN				Description
 *
 * SA0 - 0		Analog in 0		Object inserted in elevator
 * SA1 - 1		Analog in 1		Elevator chair count
 * SA2 - 2		Analog in 2		Object delivered from lift
 * SA3 - 3		Analog in 3		Object on magnet at outer end-point
 * SA4 - 4		Analog in 4		Drone docking observation
 *
 * SD0 - X		Digital in 9	Magnet end-point inner
 * SD1 - X		Digital in 8	Magnet end-point outer
 *
 */

#include "definitions.h"

// Sensor pull update time
#define CHECK_CYCLE_TIME 100

// Analog sensor pin layout
//							SA0	SA1	SA2	SA3	SA4
static char SA_CH[5] = {	0,	1,	2,	3,	4	};

// Digital sensor pin layout
//							SD0	SD1
static char SD_pins[2] = { 	9,	8	};

// Analog voltage thresholds
//								SA0	SA1	SA2	SA3	SA4
static int SA_threshold[5] = {	850,850,850,700,850	};

// Number of channels to install
#define NUM_CH 5

// Channel mapping
#define SA0 0
#define SA1 1
#define SA2 2
#define SA3 3
#define SA4 4
#define SD0 0
#define SD1 1

static int distance_check (int pin, int threshold)
{
	// Read voltage on distance sensor
	if (adcRead(pin) > threshold)
	{
		return 1;
	}
	return 0;
}

static char analog_sensor(int sensor_number)
{
	static int distance[5], was_distance[5];

	distance[sensor_number] = distance_check(SA_CH[sensor_number], SA_threshold[sensor_number]);

	if (distance[sensor_number] && (distance[sensor_number] != was_distance[sensor_number]))
	{
		was_distance[sensor_number] = distance[sensor_number]; // 1
		return 1;
	}
	else if (!distance[sensor_number] && (distance[sensor_number] != was_distance[sensor_number]))
	{
		was_distance[sensor_number] = distance[sensor_number]; // 0
		return 2;
	}
	return 0;
}

static char digital_sensor(int sensor_number)
{
	static int current_status[2], was_status[2];

	current_status[sensor_number] = digitalRead_328(SD_pins[sensor_number]);

	if (current_status[sensor_number] && (current_status[sensor_number] != was_status[sensor_number]))
	{
		was_status[sensor_number] = current_status[sensor_number];
		return 1;
	}
	was_status[sensor_number] = current_status[sensor_number];
	return 0;
}

void sensormonitor(void)
{
	static int init = 0;
	static Timer check_timer;

	// Initialize
	if (!init)
	{
		if (DEBUG && INIT_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - init\r\n");

		check_timer = newTimer_328();
		startTimer_328(CHECK_CYCLE_TIME, check_timer);
		//if (DEBUG) debug_msg(7); // "Sensor monitor - init";
		adcInstall(NUM_CH);
		pinMode_328(SD_pins[SD0], INPUT);
		pinMode_328(SD_pins[SD1], INPUT);

		init = 1;
	}

	// Sensor check
	if (timeout_328(check_timer))
	{
		startTimer_328(CHECK_CYCLE_TIME, check_timer);

		switch(analog_sensor(SA0))
		{
			case 0:
				// No change
				break;
			case 1:
				if (DEBUG && SENSOR_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - Object on elevator\r\n");
				sendSignal(object_elevator);
				break;
			case 2:
				// No importance
				break;
		}
		switch(analog_sensor(SA1))
				{
					case 0:
						// No change
						break;
					case 1:
						if (DEBUG && SENSOR_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - Elevator count\r\n");
						sendSignal(elevator_count);
						break;
					case 2:
						// No importance
						break;
				}

		switch(analog_sensor(SA2))
		{
			case 0:
				// No change
				break;
			case 1:
				if (DEBUG && SENSOR_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - Object arrived shelf\r\n");
				sendSignal(object_arrived_objectshelf_lift);
				sendSignal(object_arrived_objectshelf_magnet);
				emptySignalQueue(object_left_objectshelf);
				break;
			case 2:
				if (DEBUG && SENSOR_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - Object left shelf\r\n");
				sendSignal(object_left_objectshelf);
				emptySignalQueue(object_arrived_objectshelf_lift);
				emptySignalQueue(object_arrived_objectshelf_magnet);
				break;
		}

		switch(analog_sensor(SA3))
		{
			case 0:
				// No change
				break;
			case 1:
				if (DEBUG && SENSOR_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - Object on magnet\r\n");
				sendSignal(object_on_magnet);
				emptySignalQueue(no_object_on_magnet);
				break;
			case 2:
				if (DEBUG && SENSOR_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - No object on magnet\r\n");
				sendSignal(no_object_on_magnet);
				emptySignalQueue(object_on_magnet);
				break;
		}

		switch(analog_sensor(SA4))
		{
			case 0:
				// No change
				break;
			case 1:
				if (DEBUG && SENSOR_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - Drone docked\r\n");
				sendSignal(drone_docked);
				break;
			case 2:
				// Removing signal if drone is gone
				emptySignalQueue(drone_docked);
				break;
		}

		switch(digital_sensor(SD0))
		{
			case 0:
				// No change
				break;
			case 1:
				if (DEBUG && SENSOR_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - Inner endpoint\r\n");
				sendSignal(magnet_inner_endpoint);
				break;
		}

		switch(digital_sensor(SD1))
		{
			case 0:
				// No change
				break;
			case 1:
				if (DEBUG && SENSOR_DEBUG) usartWriteString_328(DEBUG_PORT, "SM - Outer endpoint\r\n");
				sendSignal(magnet_outer_endpoint);
				break;
		}
	}
} // end sensormonitor
