/*
 * Depot process
 *
 */

#include "definitions.h"

// Pin layout
#define MAGNET_PIN 6
#define SOLENOID_PIN 7

// Waiting periods between state changes
#define WAIT_TIME_SMALL 1500
#define WAIT_TIME_DRONE 3000

// Turn on solenoids
static void solenoids_on (void)
{
	//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Solenoids on\r\n");
	digitalWrite_328(SOLENOID_PIN, HIGH);
}

// Turn off solenoids
static void solenoids_off (void)
{
	//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Solenoids off\r\n");
	digitalWrite_328(SOLENOID_PIN, LOW);
}

// Turn on electromagnet
static void magnet_on (void)
{
	//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Magnet on\r\n");
	digitalWrite_328(MAGNET_PIN, HIGH);
}

// Turn off electromagnet
static void magnet_off (void)
{
	//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Magnet off\r\n");
	digitalWrite_328(MAGNET_PIN, LOW);
}

void depot(void)
{
	static enum { waiting, up_position, down_position } position;
	static char order;
	static SwitchType arm_holding;
	static Timer t1;
	static char init = 0;

	// Initialize
	if(!init)
	{
		//if (DEBUG && INIT_DEBUG) usartPrint_328(DEBUG_PORT, "D - init\r\n");
		pinMode_328(MAGNET_PIN, OUTPUT);
		pinMode_328(SOLENOID_PIN, OUTPUT);

		// Default start configuration: arm raised, magnet off, solenoids off
		magnet_off();
		solenoids_off();
		t1 = newTimer_328();

		position = waiting;
		order = 1;
		emptySignalQueue(arm_endpoint_up);
		emptySignalQueue(arm_endpoint_down);
		sendSignal(arm_up);

		init = 1;
	}

	// Sequence of events to keep the arm moving
	switch(position)
	{
		case waiting:
			if(receivedSignal(arm_is_up))
			{
				//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Waiting, received arm_is_up\r\n");
				solenoids_on();
				startTimer_328(WAIT_TIME_SMALL, t1);
				position = up_position;

			}
			else if(receivedSignal(arm_is_down))
			{
				//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Waiting, received arm_is_down\r\n");
				startTimer_328(WAIT_TIME_SMALL, t1);
				position = down_position;
			}
			break;

		case up_position:
			switch(order)
			{
				case 1:
					if(timeout_328(t1))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Up - 1/7 Arm up \r\n");
						arm_holding = off;
						//sendSignal(arm_hold, &arm_holding);
						order = 2;
					}
					break;

				case 2:
					if(receivedSignal(drone_docked))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Up - 2/7 Drone docked\r\n");
						magnet_on();
						order = 3;
					}
					break;

				case 3:
					if(receivedSignal(object_on_magnet))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Up - 3/7 Object on magnet\r\n");
						order = 4;
					}
					break;

				case 4:
					if (receivedSignal(no_drone_docked))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Up - 4/7 Drone gone\r\n");
						startTimer_328(WAIT_TIME_DRONE,t1);
						order = 5;
					}
					break;

				case 5:
					if(timeout_328(t1))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Up - 5/7 Activate stepper motor to hold arm outright\r\n");
						arm_holding = on;
						//sendSignal(arm_hold, &arm_holding);
						startTimer_328(WAIT_TIME_SMALL, t1);
						order = 6;
					}
					break;

				case 6:
					if(timeout_328(t1))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Up - 6/7 Disengage solenoid holding bolts\r\n");
						solenoids_off();
						startTimer_328(WAIT_TIME_SMALL,t1);
						order = 7;
					}
					break;

				case 7:
					if(timeout_328(t1))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Up - 7/7 Move arm down\r\n");
						sendSignal(arm_down);
						emptySignalQueue(arm_endpoint_down);
						order = 1;
						position = waiting;
					}
					break;
			}
			break;

		case down_position:
			switch(order)
			{
				case 1:
					if(timeout_328(t1))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Down - 1/3 Disengage magnet, drop object\r\n");
						magnet_off();
						order = 2;
					}
					break;

				case 2:
					if(receivedSignal(no_object_on_magnet))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Down - 2/3 No object on magnet\r\n");
						startTimer_328(WAIT_TIME_SMALL, t1);
						order = 3;
					}
					break;

				case 3:
					if(timeout_328(t1))
					{
						//if (DEBUG && DEPOT_DEBUG) usartPrint_328(DEBUG_PORT, "D - Down - 3/3 Move arm up\r\n");
						sendSignal(arm_up);
						emptySignalQueue(arm_endpoint_up);
						order = 1;
						position = waiting;
					}
					break;

				default:
					break;
			}
			break;
	}
} // end depot
