/*
 * Sensor monitor process
 *
 * Sensors   S - Sensor, A - Analog, D - Digital, # - Number
 *
 * Code Channel PIN				Description
 *
 * SD0 - 0		Digtal pin 8	Arm is up sensor
 * SD1 - 1		Digtal pin 9	Arm is down sensor
 * SA0 - 0		Analog in 0		Object is on magnet sensor
 * SA1 - 1		Analog in 1		Drone dock sensor
 *
 */

 #include "definitions.h"

// Number of channels to install
#define NUM_CH 4 //2 is the original, 4 for test rig

// Analog sensor pin layout
//						SA0	SA1 SA2 SA3
static char SA_CH[] = {	0,	2,  1,  3  };

// Digital sensor pin layout
//							SD0	SD1
static char SD_pins[] = { 	8,	9	};

//digital sensor replacement for test rig.
// uses analog sensors instead SA2 and SA3

// Analog voltage thresholds
//							 SA0  SA1  SA2  SA3
static int SA_threshold[] = {750, 750, 500, 500 };

// Channel mapping
#define SA0 0
#define SA1 1
#define SD0 0
#define SD1 1

//test rig
#define SA2 2
#define SA3 3

// Sensor pull update time
#define CHECK_CYCLE_TIME 100

static unsigned char distance_check (char sensor_number)
{
	// Read voltage on distance sensor
	if (adcRead(SA_CH[(int)sensor_number]) > SA_threshold[(int)sensor_number])
	{
		return 1;
	}
	return 0;
}

static unsigned char analog_sensor(char sensor_number)
{
	static char distance[4], was_distance[4];

	distance[(int)sensor_number] = distance_check(sensor_number);

	if (distance[(int)sensor_number] && (distance[(int)sensor_number] != was_distance[(int)sensor_number]))
	{
		was_distance[(int)sensor_number] = distance[(int)sensor_number]; // 1
		return 1;
	}
	else if ((distance[(int)sensor_number] != was_distance[(int)sensor_number]))
	{
		was_distance[(int)sensor_number] = distance[(int)sensor_number]; // 0
		return 2;
	}
	return 0;
}

static char digital_sensor(char sensor_number)
{
	static char current_status[2], was_status[2] = {0,0};

	current_status[(int)sensor_number] = digitalRead_328(SD_pins[(int)sensor_number]);

	if (current_status[(int)sensor_number] && (current_status[(int)sensor_number] != was_status[(int)sensor_number]))
	{
		was_status[(int)sensor_number] = current_status[(int)sensor_number];
		return 1;
	}
	was_status[(int)sensor_number] = current_status[(int)sensor_number];
	return 0;
}

void sensormonitor(void)
{
	static char init = 0;
	static Timer check_timer;

	// Initialize
	if (!init)
	{
		//if (DEBUG && INIT_DEBUG) usartPrint_328(DEBUG_PORT, "SM - init\r\n");
		check_timer = newTimer_328();
		startTimer_328(CHECK_CYCLE_TIME, check_timer);
		adcInstall(NUM_CH);
		//pinMode_328(SD_pins[SD0], INPUT);
		//pinMode_328(SD_pins[SD1], INPUT);

		init = 1;
	}

	// Sensor check
	if (timeout_328(check_timer))
	{
		startTimer_328(CHECK_CYCLE_TIME, check_timer);
		/*
		 * Test rig, uses analog sensors
		switch(digital_sensor(SD0))
		{
			case 0:
				// No change
				break;
			case 1:
				//if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Arm enpoint up\r\n");
				sendSignal(arm_endpoint_up);
				break;
		}

		switch(digital_sensor(SD1))
		{
			case 0:
				// No change
				break;
			case 1:
				//if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Arm enpoint down\r\n");
				sendSignal(arm_endpoint_down);
				break;
		}
		*/
		switch(analog_sensor(SA0))
		{
			case 0:
				// No change
				break;
			case 1:
				//if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Object on magnet\r\n");
				sendSignal(object_on_magnet);
				emptySignalQueue(no_object_on_magnet);
				break;
			case 2:
				//if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - No object on magnet\r\n");
				sendSignal(no_object_on_magnet);
				emptySignalQueue(object_on_magnet);
				break;
		}

		switch(analog_sensor(SA1))
		{
			case 0:
				// No change
				break;
			case 1:
				//if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Drone docked\r\n");
				sendSignal(drone_docked);
				emptySignalQueue(no_drone_docked);
				break;
			case 2:
				//if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - No drone docked\r\n");
				sendSignal(no_drone_docked);
				emptySignalQueue(drone_docked);
				break;
		}

		// SA2 and SA3 are analoge sensors for test rig only. they replace the digital sensors i
		// in the original rig
		switch(analog_sensor(SA2))
		{
			case 0:
				// No change
				break;
			case 1:
				//gone high
				//if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Arm enpoint up\r\n");
				sendSignal(arm_endpoint_up);
				break;
			case 2:
				//gone low
				emptySignalQueue(arm_endpoint_up);
				break;
		}

		switch(analog_sensor(SA3))
		{
			case 0:
				// No change
				break;
			case 1:
				// gone high
				//if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Arm enpoint down\r\n");
				sendSignal(arm_endpoint_down);
				break;
			case 2:
				//gone low
				emptySignalQueue(arm_endpoint_down);
				break;
		}
	}
} // end sensormonitor
