/*
 * Stepper motor process for the arm
 *
 * Full-step - more strength and torque
 *  A single step is 1.8deg (cycle) consisting of a total of 4 operations, one revolution 200 steps
 * Half-step - accurate, mix of wave drive and full step mode
 *  A single step is 0.9deg (cycle) consisting of a total of 8 operations, one revolution 400 step
 *
 * char steps[] = {0b1000,0b0100,0b0010,0b0001}; // wavedrive
 * char steps[] = {0b1001,0b1010,0b0110,0b0101}; // full-step
 * char steps[] = {0b1000,0b1010,0b0010,0b0110,0b0100,0b0101,0b0001,0b1001}; // half-step
 *
 */

#include "definitions.h"

// Timing delay to adjust the stepper motor speed
#define ARM_SPEED 50

void stepper_arm(void)
{
	static unsigned char stepper_motorpins[] = {10, 11, 12, 13};
	static int steps[] = {0b0101, 0b0110, 0b1010, 0b1001}; // full-step
	static int seq[] = {0b1000, 0b0100, 0b0010, 0b0001}; // a single cycle sequence
	static enum {still, up, down} current_operation = still;
	static SwitchType arm_holding;

	static int i = 0;
	static int last_i;
	static Timer t1;
	static char init = 0;

	// Initialize
	if (!init)
	{
		//if (DEBUG && INIT_DEBUG) usartPrint_328(DEBUG_PORT, "S - init\r\n");
		t1 = newTimer_328();
		for (int j = 0; j < 4; j++) // pins
		{
			pinMode_328(stepper_motorpins[j], OUTPUT);
			digitalWrite_328(stepper_motorpins[j], 0);
		}
		init = 1;
	}

	switch (current_operation)
	{
		case still:

			if (receivedSignal(arm_up))
			{
				//if (DEBUG && STEPPER_DEBUG) usartPrint_328(DEBUG_PORT, "S - Still - Received arm_up\r\n");
				startTimer_328(ARM_SPEED, t1);
				current_operation = up;
			}
			else if (receivedSignal(arm_down))
			{
				//if (DEBUG && STEPPER_DEBUG) usartPrint_328(DEBUG_PORT, "S - Still - Received arm_down\r\n");
				startTimer_328(ARM_SPEED, t1);
				current_operation = down;
			}
			//wait a little while for motor to set pins to 0,0,0,0
			else if (receivedSignal(arm_hold, &arm_holding))
			{
				switch(arm_holding)
				{
					case off:
						//if (DEBUG && STEPPER_DEBUG) usartPrint_328(DEBUG_PORT, "S - Still - Received arm holding off\r\n");
						for (int j = 0; j < 4; j++) // pins
						{
							digitalWrite_328(stepper_motorpins[j], 0);
						}
						break;

					case on:
						//if (DEBUG && STEPPER_DEBUG) usartPrint_328(DEBUG_PORT, "S - Still - Received arm holding on\r\n");
						for (int j = 0; j < 4; j++) // pins
						{
							digitalWrite_328(stepper_motorpins[j], (steps[last_i] & seq[j]));
						}
						break;
				}
			}

			break;

		case up:
			if (receivedSignal(arm_endpoint_up))
			{
				//if (DEBUG && STEPPER_DEBUG) usartPrint_328(DEBUG_PORT, "S - Up - Received endpoint up\r\n");
				// aetype tells which sensor triggered
				current_operation = still;
				sendSignal(arm_is_up);
				emptySignalQueue(arm_is_down);
			}
			// Continuously keep moving the motor
			else if (timeout_328(t1))
			{

				startTimer_328(ARM_SPEED, t1);
				for (int j = 0; j < 4; j++) // pins
				{
					digitalWrite_328(stepper_motorpins[j], (steps[i] & seq[j]));
				}
				last_i = i;
				if (++i > (4 - 1)) i = 0;
			}
			break;

		case down:
			if (receivedSignal(arm_endpoint_down))
			{
				//if (DEBUG && STEPPER_DEBUG) usartPrint_328(DEBUG_PORT, "S - Down - Received endpoint down\r\n");
				// aetype tells which sensor triggered
				current_operation = still;
				sendSignal(arm_is_down);
				emptySignalQueue(arm_is_up);
			}

			// Continuously keep moving the motor
			else if (timeout_328(t1))
			{
				startTimer_328(ARM_SPEED, t1);
				for (int j = 0; j < 4; j++) // pins
				{
					digitalWrite_328(stepper_motorpins[j], (steps[i] & seq[j]));
				}
				last_i = i;
				if (--i < 0) i = (4 - 1);
			}
			break;
	}
} // end stepper
