/*
 * Sensor monitor process
 *
 * Sensors   S - Sensor, A - Analog, D - Digital, # - Number
 *
 * Code Channel PIN				Description
 *
 * SA0 - 0		Analog in 0		Sense 1
 * SA1 - 1		Analog in 1		Sense 2
 * SA2 - 2		Analog in 2		Sense 3
 * SA3 - 3		Analog in 3		Sense 4
 *
 */
#include "definitions.h"

// Sensor pull update time
#define CHECK_CYCLE_TIME 100

// Number of channels to install
#define NUM_CH 4

// Analog sensor pin layout
//							SA0	SA1	SA2	SA3
static char SA_CH[4] = {	0,	1,	2,	3	};

// Analog voltage thresholds
#define THRESHOLD_LOW 102 // Approx. 0.5V
#define THRESHOLD_DRONE_GONE 102  // Approx. 1V and lower to detect that the drone has left/disconnected

// Channel mapping
#define SA0 0
#define SA1 1
#define SA2 2
#define SA3 3

static int voltage_check_low(char ch_num)
{
	static int current;

	current = adcRead(ch_num);
	usartPrint_328(DEBUG_PORT, "SM - Check - value: %d\r\n",current);
	if (current > THRESHOLD_LOW)
	{
		return 1;
	}
	return 0;

}

static char drone_gone(char ch_num)
{
	static int current;

	current = adcRead(ch_num);

	if(current > THRESHOLD_DRONE_GONE)
	{
		return 1;
	}
	return 0;
}

void sensormonitor(void)
{
	static char init = 0;
	static Timer check_timer;
	static char voltage[4], last_voltage[4] ={0, 0, 0, 0};
	static char drone_connected_flag;
	static char charge_pin_count;
	static char prell_removal_count = 0;

	// Initialize
	if (!init)
	{
		if (DEBUG && INIT_DEBUG) usartPrint_328(DEBUG_PORT, "SM - init\r\n");
		check_timer = newTimer_328();
		startTimer_328(CHECK_CYCLE_TIME, check_timer);
		adcInstall(NUM_CH);
		drone_connected_flag = 0;
		charge_pin_count = 0;
		init = 1;
	}

	// Sensor check
	if (timeout_328(check_timer))
	{
		startTimer_328(CHECK_CYCLE_TIME, check_timer);

		if(!drone_connected_flag)
		{
			for(int i = 0; i<4; i++)
			{
				voltage[i] = voltage_check_low(SA_CH[i]);
				if (voltage[i])
				{
					charge_pin_count++;
				}
				last_voltage[i] = voltage[i];
			}
			if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Check - %d,%d,%d,%d\r\n",voltage[0], voltage[1], voltage[2], voltage[3]);
			if (charge_pin_count == 3)
			{
				prell_removal_count++;
				if (DEBUG && INIT_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Check - Prell++ \r\n");
			}
			else
			{
				if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Check - Prell reset\r\n");
				prell_removal_count = 0;
			}
			charge_pin_count = 0;

			if (prell_removal_count >= 20)
			{
				if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Check - Conected\r\n");
				drone_connected_flag = 1;
				prell_removal_count = 0;
				sendSignal(drone_connected, &voltage[0], &voltage[1], &voltage[2], &voltage[3]);
			}
		}
		else if (drone_connected_flag)
		{
			for(int i = 0; i<4; i++)
			{
				voltage[i] = drone_gone(SA_CH[i]);

				if (!voltage[i] && (voltage[i] != last_voltage[i]))
				{
					charge_pin_count++;
				}
			}
			if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Connected -  charge pin connected %d\r\n",charge_pin_count);
			if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Connected -  is %d,%d,%d,%d - was %d,%d,%d,%d\r\n"
					,voltage[0], voltage[1], voltage[2], voltage[3],last_voltage[0], last_voltage[1], last_voltage[2], last_voltage[3]);
			if (charge_pin_count)
			{
				sendSignal(done_charging);
				drone_connected_flag = 0;
				if (DEBUG && SENSOR_DEBUG) usartPrint_328(DEBUG_PORT, "SM - Connected -  lost connection\r\n");
			}

			charge_pin_count = 0;
		}
	}
} // end sensormonitor


