 #include "usarts_328.h"
#include <avr/io.h>
#include <stdlib.h>
#include <stdio.h>
#include <avr/interrupt.h>

#define TX_BUFFER_SIZE 64
#define RX_BUFFER_SIZE 64


//########################### USARTS ##############################

typedef struct
{
	int 	length;    	// data buffer length
	int 	n_elements; // number of elements in buffer
	int  	inpt;  		// write pointer
	int  	outpt;		// read pointer
	char	buff[];   	// the buffer
} fifo_t, * Fifo;


static Fifo TXfifo[] = {NULL};
static Fifo RXfifo[] = {NULL};
static int usart_installed[] = {0};


static Fifo newFifo(unsigned int length)
{
	Fifo fifo;
	fifo = (Fifo)malloc(sizeof(fifo_t)+length);
	fifo -> length = length;
	fifo -> n_elements = 0;
	fifo -> inpt = 0;
	fifo -> outpt = 0;
	return fifo;
}

// see if buffer is empty
static int fifoEmpty(Fifo fifo)
{
	return (fifo -> n_elements == 0);
}

// see if buffer is full
static int fifoFull(Fifo fifo)
{
	return (fifo -> n_elements >= fifo -> length);
}


// put a char into fifo
static int fifoPut(Fifo fifo, char c)
{
	int ok;
	unsigned char old_SREG = SREG;
	cli();
	if (!fifoFull(fifo))
	{
		fifo -> n_elements++;
		fifo -> buff[fifo -> inpt++] = c;
		if (fifo -> inpt >= fifo -> length)
			fifo -> inpt = 0;
		ok = 1;
	}
	else
		ok = 0;
	SREG = old_SREG;
	return ok;
}

// get a char from fifo
static int fifoGet(char *c, Fifo fifo)
{
	int ok = 0;
	unsigned char old_SREG = SREG;
	cli();
	if (!fifoEmpty(fifo))
	{
		fifo -> n_elements--;
		*c = fifo -> buff[fifo -> outpt++];
		if (fifo -> outpt >= fifo -> length)
			fifo -> outpt = 0;
		ok = 1;
	}
	else
		*c = '\0';
	SREG = old_SREG;
	return ok;
}


SIGNAL(USART_RX_vect)
{
	char data = UDR0; // possible something else than UDR0
	if (!fifoFull(RXfifo[0]))
		fifoPut(RXfifo[0], data);
}

SIGNAL(USART_TX_vect)
{
	char c;
	if (fifoGet(&c, TXfifo[0]))
	{
		UDR0 = c; // possible something else than UDR0
	}
}

// maybe remove port, since there are only one and use 0 as static
static int usartTXdataEmpty(int port)
{
	switch(port)
	{
		case 0:
			return ((UCSR0A & (1<<UDRE0))!=0);
		default:
			return 0;
	}
}


// maybe remove port, since there are only one and use 0 as static
void usartWrite_328(int port, char c)
{
	if (port > 0)
		return;
	if(!usart_installed[port])
		return;
	if (fifoEmpty(TXfifo[port]) && usartTXdataEmpty(port))
	{
		//first character is sent directly to usart
		switch (port)
		{
			case 0: UDR0 = c; break;
		}
	}
	else // character is sent to fifo
		while (!fifoPut(TXfifo[port], c));
}

 // maybe remove port, since there are only one and use 0 as static
int usartDataIsPresent_328(int port)
{
	if (port > 0)
		return 0;
    if(!usart_installed[port])
    	return 0;
	return (!fifoEmpty(RXfifo[port]));
}

char usartRead_328(int port)
{
	char c;
	if (port > 0)
		return 0;
	if(!usart_installed[port])
		return 0;
	while (!fifoGet(&c, RXfifo[port]));
    return c;
}

void usartInstall_328(int port, unsigned long baud )
{
    unsigned int rate;
    if (port > 0)
    	return;
    if (usart_installed[port])
    	return;
    //check legal baudrate
    switch(baud)
    {
        case   1200: break;
        case   2400: break;
        case   4800: break;
        case   9600: break;
        case  14400: break;
        case  19200: break;
        case  38400: break;
        case  56000: break;
        case  57600: break;
        case  76800: break;
        case 115200: break;
        default: baud = 9600; break;
    }
    rate = ((unsigned int)(((unsigned long)F_CPU/baud)/(unsigned long)8))-1;
    switch (port)
    {
		case 0:
		    // Set baud rate
		    UBRR0 = rate;
			UCSR0A = (1<<U2X0);
		    // Enable receiver and transmitter
		    UCSR0B = (1<<RXCIE0)|(1<<TXCIE0)|(1<<RXEN0)|(1<<TXEN0);
		    // Set frame format: 8 data, 1stop bit
		    UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);
			break;
    }
    RXfifo[port] = newFifo(RX_BUFFER_SIZE);
	TXfifo[port] = newFifo(TX_BUFFER_SIZE);
    usart_installed[port] = 1;
    sei();
}

void usartWriteString_328(int port, char string[])
{
    unsigned char i = 0;
	if (port > 0)
		return;
    while (string[i] != '\0')
		usartWrite_328(port,string[i++]);
}


static void usartPutChar0(char c)
{
	usartWrite_328(0,c);
}

void usartPrint_328(int port, char *format, ...)
{
	if (port > 0)
		return;
	if(!usart_installed[port])
		return;

	FILE* stream = NULL;;
	switch (port)
	{
		case 0:
			stream = fdevopen(usartPutChar0,NULL);
			break;
	}
    va_list ap;
    va_start(ap,format);
    vfprintf (stream,format,ap);
    fclose(stream);
}


